#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <pipethrough.h>

static int writen(int fd, const void *buf, size_t len)
	/*@globals errno; @*/
	/*@modifies errno; @*/
{
	ssize_t res;
	while (len > 0) {
		if ((res = write(fd, buf, len)) == (ssize_t) -1) {
			return -1;
		}
		buf = &(((const char *) buf)[(size_t) res]);
		len -= (size_t) res;
	}
	return 0;
}

int main(int argc, char **argv)
	/*@globals internalState, fileSystem, errno, stdout, stderr; @*/
	/*@modifies internalState, fileSystem, errno, *stdout, *stderr; @*/
{
	struct pipe_inbuffer in = { "hello, world\n", (size_t) 13 };
	struct pipe_outbuffer out, err;
	int status;

	if (argc != 2) {
		(void) fprintf(stderr, "usage: pipethrough <shell-command>\n");
		exit(EXIT_FAILURE);
	}
	if (pipethrough(argv[1], &in, &out, &err, &status) == -1) {
		perror("pipethrough");
		exit(EXIT_FAILURE);
	}
	(void) fprintf(stdout, "%d %lu %lu\n", status, (unsigned long) out.size, (unsigned long) err.size);
	if (out.data != NULL) {
		if (writen(STDOUT_FILENO, out.data, out.size) == -1) {
			perror("writen");
			exit(EXIT_FAILURE);
		}
	}
	if (err.data != NULL) {
		if (writen(STDERR_FILENO, err.data, err.size) == -1) {
			perror("writen");
			exit(EXIT_FAILURE);
		}
	}

	pipe_outbuffer_finalize(&out);
	pipe_outbuffer_finalize(&err);
	return 0;
}
